/*
 * LibSKK, a tiny Library to emulate SKK (Simple Kana Kanji Conversion)
 * 
 * Copyright (C) 2002 Motonobu Ichimura <famao@kondara.org>
 *
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, and/or sell copies of the Software, and to permit persons
 * to whom the Software is furnished to do so, provided that the above
 * copyright notice(s) and this permission notice appear in all copies of
 * the Software and that both the above copyright notice(s) and this
 * permission notice appear in supporting documentation.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT
 * OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
 * HOLDERS INCLUDED IN THIS NOTICE BE LIABLE FOR ANY CLAIM, OR ANY SPECIAL
 * INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Except as contained in this notice, the name of a copyright holder
 * shall not be used in advertising or otherwise to promote the sale, use
 * or other dealings in this Software without prior written authorization
 * of the copyright holder.
 *
 */

/* $Id: skkcursor.c,v 1.3 2003/02/19 17:44:51 famao Exp $ */

/* vi:set ts=4 sw=4: */


#include "skkcursor.h"

SkkCursor*
skk_cursor_new (void)
{
	SkkCursor *ret;
	ret = g_new0 (SkkCursor, 1);
	ret->buf = NULL;
	ret->separator = g_strdup ("|");
	return ret;
}

void
skk_cursor_destroy (SkkCursor *cursor)
{
	if (!cursor)
		return;
	skk_cursor_clear (cursor);
	if (cursor->separator) {
		g_free (cursor->separator);
		cursor->separator = NULL;
	}
	g_free (cursor);
	return;
}

void
skk_cursor_set_separator (SkkCursor *cursor, const gchar *sep)
{
	if (!cursor || !sep)
		return;
	if (cursor->separator) {
		g_free (cursor->separator);
		cursor->separator = NULL;
	}
	cursor->separator = g_strdup (sep);
	return;
}

void
skk_cursor_set_buf (SkkCursor *cursor, const gchar *buf)
{
	if (!cursor)
		return;
	if (cursor->buf) {
		g_free (cursor->buf);
		cursor->buf = NULL;
	}
	if (!buf)
		return;
	cursor->buf = g_strdup (buf);
	return;
}

void
skk_cursor_clear (SkkCursor *cursor)
{
	if (!cursor)
		return;
	if (cursor->buf) {
		g_free (cursor->buf);
		cursor->buf = NULL;
	}
	cursor->revPos = 0;
	cursor->revLen = 0;
	cursor->caret = 0;
	return;
}
